#include <iomanip>
#include <vector>
#include <random>
#include <cmath>
#include <cpu_timer.h>
#include <la_operations.h>

int main(int /*argc*/, char **/*argv[]*/)
{
    // create cpu-timer
    CPUTimer timer;

    // just set up a random number generator(Mersenne twister)
    std::random_device dev;
    std::mt19937 gen(dev());
    std::uniform_real_distribution<> dis(0.0, 1.0); // https://en.cppreference.com/w/cpp/numeric/random/uniform_real_distribution

    for(unsigned int m_exp = 2; m_exp <= 12; m_exp++) {

        unsigned int m = (int) std::pow(2,m_exp);

        std::cout << "performing copy-benchmark at m=" << m << std::endl;

        // we initialize matrix first and only after measure copy-runtime to avoid measuring implementational overhead
        la_objects::LABaseObject<double> A(m, m);
        for (unsigned int r = 0; r < A.n_rows(); r++)
        {
            for (unsigned int c = 0; c < A.n_cols(); c++)
            {
                A(r, c) = dis(gen);
            }
        }

        // create target matrix 
        la_objects::LABaseObject<double> B(m, m);

        // now measure timings for matrix copy
        double avg_timing = timer.benchmark([&]() {la_operations::copy_data(A, B);}, 100, 10);

        // test successful assignment
        for (unsigned int r = 0; r < A.n_rows(); r++)
        {
            for (unsigned int c = 0; c < A.n_cols(); c++)
            {
                if (!(A(r, c) == B(r, c))) {
                    throw std::runtime_error("test failed");
                }
            }
        }

        // dump timing to err-stream
        std::cerr << m << " " << timer.elapsed() << std::endl;
    }

    return 0;
}

